/*
         Copyright (c) 2003, WebThing Ltd
         Author: Nick Kew <nick@webthing.com>
                 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "validator.h"

#include <stdio.h>
#include <stdlib.h>


GtkEntry* url_widget ;

void
on_browse_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* w = create_fileselection1() ;
  url_widget = GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "url")) ;
  gtk_widget_show(w) ;
}

static parsemode get_parsemode(GtkWidget* w) {
  typedef struct {
    const gchar* const name ;
    parsemode value ;
  } parse_button ;
  static parse_button options[] = {
	{ "html" , p_html } ,
	{ "sgml" , p_sgml } ,
	{ "fussy" , p_fussy } ,
	{ "xml" , p_xml } ,
	{ NULL, p_html }
  } ;
  parse_button* item;
  for (item = options ; item->name != NULL ; ++item) {
    GtkWidget* radio_menu_item = lookup_widget ( w , item->name ) ;
    if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(radio_menu_item) ) )
      return item->value ;
  }
  return p_html ;        // shut compiler up
}
static void errmsg(GtkWidget* w, const char* message) {
  static int ctx_id = 0 ;
  static int msg_id = 0 ;
  GtkStatusbar* bar = GTK_STATUSBAR(lookup_widget(w, "statusbar1")) ;
  ctx_id = gtk_statusbar_get_context_id(bar, message) ;
  msg_id = gtk_statusbar_push(bar, ctx_id, message) ;
}

void
on_ok_clicked                          (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* w = GTK_WIDGET(button) ;
  parsemode mode = get_parsemode(w) ;
  GtkCList* results = GTK_CLIST(lookup_widget(w, "results")) ;
  gchar* url = 0 ;
  gchar* enc = 0 ;
  const char* error ;

  GtkLabel* encl = GTK_LABEL(GTK_BIN(lookup_widget(w, "encoding"))->child) ;
  gtk_label_get(encl, &enc) ;

  url_widget = GTK_ENTRY(lookup_widget(w, "url") ) ;
  url = gtk_entry_get_text(url_widget) ;




  if ( error = validate(url, mode, results, enc), error)
    errmsg(w, error) ;
}


void
on_help_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
  char buf[256] ;
  const char* base = getenv("VALET_BASE") ;
  const char* browser = getenv("BROWSER") ;
  GtkWidget* w = GTK_WIDGET(button) ;
  if ( ! base ) {
    errmsg(w, "Environment variable VALET_BASE is unset") ;
    return ;
  }
  if ( ! browser ) {
    browser = "mozilla" ;
    errmsg(w, "Environment variable BROWSER is unset; using mozilla") ;
  }
  sprintf(buf, "%s %s/html/validator.html &", browser, base) ;
  system(buf) ;
}


void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* selector = lookup_widget(GTK_WIDGET(button), "fileselection1") ;
  gchar* value = gtk_file_selection_get_filename
	  ( GTK_FILE_SELECTION(selector) );
  gtk_entry_set_text(url_widget, value) ;
  gtk_widget_destroy(selector) ;
}

