/*
         Copyright (c) 2003, WebThing Ltd
         Author: Nick Kew <nick@webthing.com>
                 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "validator.h"

#include <OpenSP/config.h>
#include <OpenSP/Boolean.h>

#include <OpenSP/ParserEventGeneratorKit.h>
#include <OpenSP/SGMLApplication.h>

class Validator : public SGMLApplication {
  GtkCList* results ;
  const char* url ;
public:
  unsigned int count ;
  Validator(GtkCList* r, const char* u) : results(r), url(u), count(0) {
    gtk_clist_clear(results) ;
  }
  void error(const ErrorEvent& event) {
    char msg[256] ;
    for (unsigned int n = 0; n < event.message.len; ++n) {
      unsigned int l = event.message.ptr[n] ;
      msg[n] = char(l) ;
    }
    char* fields[4] ;
    msg[event.message.len+1] = 0 ;
    char* colon = strstr(msg, url) ;    /* jump over url */
    if ( !colon )
    	    return ;
    char* l = colon + strlen(url) ;
    if ( colon = strchr(l, ':') , !colon )
    	    return ;
    l = colon+1;                        /* line */
    if ( colon = strchr(l, ':') , !colon )
    	    return ;
    *colon = 0 ;
    fields[1] = l ;
    l = colon+1;                        /* col */
    if ( colon = strchr(l, ':') , !colon )
    	    return ;
    *colon = 0 ;
    fields[2]  = l ;
    l = colon+1;                        /* severity */
    if ( colon = strchr(l, ':') , !colon )
    	    return ;
    *colon = 0 ;
    fields[0]  = l ;
    l = colon+1;                        /* message */
    if ( colon = strchr(l, '\n') , colon )
    	    *colon = 0 ;
    fields[3]  = l ;
    gtk_clist_append(results, fields) ;
    ++count ;
  }
} ;


const char* validate(gchar* url, parsemode mode, GtkCList* results, gchar* enc) {

  setenv("SP_MESSAGE_FORMAT", "TRADITIONAL", 1) ;

  const char* htmlCatalogue = getenv("HTML_CATALOGUE") ;
  if ( !htmlCatalogue )
    return "Missing environment variable HTML_CATALOGUE" ;
  const char* xmlCatalogue = getenv("XML_CATALOGUE") ;
  if ( !xmlCatalogue )
    return "Missing environment variable XML_CATALOGUE" ;
  
  if ( mode == p_xml )
    setenv("SP_ENCODING", "xml", 1) ;
  else if ( strcmp(enc, "Unknown" ) )
    setenv("SP_ENCODING", enc, 1) ;
  else
    unsetenv("SP_ENCODING") ;

  ParserEventGeneratorKit parserKit ;
  switch ( mode ) {
    case p_sgml:
      setenv("SGML_CATALOG_FILES", htmlCatalogue , 1) ;
      break ;
    case p_html:
      setenv("SGML_CATALOG_FILES", htmlCatalogue , 1) ;
      parserKit.setOption(ParserEventGeneratorKit::enableWarning, "min-tag") ;
      break ;
    case p_fussy:
      setenv("SGML_CATALOG_FILES", htmlCatalogue , 1) ;
      parserKit.setOption(ParserEventGeneratorKit::enableWarning, "min-tag") ;
      parserKit.setOption(ParserEventGeneratorKit::enableWarning, "fully-tagged") ;
      break ;
    case p_xml:
      setenv("SGML_CATALOG_FILES", xmlCatalogue , 1) ;
      break ;
  }
  Validator val(results, url) ;
  EventGenerator *egp = parserKit.makeEventGenerator(1, &url);
  egp->inhibitMessages(true) ;
  int nErrors = egp->run(val) ;
  delete egp ;
  static char buf[80] ;
  if ( val.count > 0 )
    sprintf(buf, "Invalid: %d errors found", val.count) ;
  else if ( nErrors > 0 )
    sprintf(buf, "Error: maybe document is unreachable or you mistyped it?") ;
  else
    sprintf(buf, "Document is valid") ;

  return buf ;
}

